SystemOrganization addCategory: #'Pier-Twitter-Model'!
SystemOrganization addCategory: #'Pier-Twitter-Seaside'!
SystemOrganization addCategory: #'Pier-Twitter-View'!

Object subclass: #PTTwitter
	instanceVariableNames: 'id'
	classVariableNames: 'StringConverter'
	poolDictionaries: ''
	category: 'Pier-Twitter-Model'!

PTTwitter subclass: #PTStatus
	instanceVariableNames: 'text created user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Twitter-Model'!

!PTStatus class methodsFor: 'accessing-description' stamp: 'lr 7/9/2008 13:45'!
descriptionCreated
	^ MAStringDescription new
		xmlName: #'created_at';
		accessor: #created;
		beRequired;
		yourself! !

!PTStatus class methodsFor: 'accessing-description' stamp: 'lr 6/11/2008 14:34'!
descriptionText
	^ MAMemoDescription new
		xmlName: #text;
		accessor: #text;
		beRequired;
		yourself! !

!PTStatus class methodsFor: 'accessing-description' stamp: 'JohnMcKeon 2/27/2010 15:04'!
descriptionUser
	^ MAToOneRelationDescription new
		classes: (Array with: PTUser);
		stringReader: MAXMLStringReader;
		xmlName: #user;
		accessor: #user;
		beRequired;
		yourself! !

!PTStatus class methodsFor: 'accessing' stamp: 'lr 6/11/2008 14:32'!
xmlName
	^ #status! !

!PTStatus methodsFor: 'private' stamp: 'JohnMcKeon 2/28/2010 08:44'!
agoString
	"Probably not that correct, maybe even entirely wrong. The goal is to be fuzzy, as much as possible. If you know better, then do better yourself and commit your changes."
	
	| duration minutes |
	self created ifNil: [ ^ nil ].
	duration :=  DateAndTime now - self created asLocal.
	minutes := duration asSeconds // 60.
	minutes <= 1
		ifTrue: [ ^ 'about one minute ago' ].
	minutes <= 50
		ifTrue: [ ^ minutes printString , ' minutes ago' ].
	minutes <= 120
		ifTrue: [ ^ 'about one hour ago' ].
	minutes <= 1080
		ifTrue: [ ^ (minutes // 60) printString , ' hours ago' ].
	minutes <= 2880
		ifTrue: [ ^ 'about one day ago' ].
	minutes <= 32000
		ifTrue: [ ^ (minutes // 1440) printString , ' days ago' ].
	minutes <= 86400
		ifTrue: [ ^ 'about one month ago' ].
	^ (minutes // 43200) printString , ' months ago'! !

!PTStatus methodsFor: 'accessing' stamp: 'lr 6/11/2008 14:39'!
created
	^ created! !

!PTStatus methodsFor: 'accessing' stamp: 'lr 7/23/2008 22:09'!
created: anObject
	created := anObject isString
		ifTrue: [ self parseTimestamp: anObject ]
		ifFalse: [ anObject ]! !

!PTStatus methodsFor: 'private' stamp: 'lr 10/13/2009 16:35'!
parseTimestamp: aString
	| token |
	token := aString findTokens: ' :'.
	^ DateAndTime
		year: (token at: 8 ifAbsent: [ Date current year ]) asInteger
		month: (Date indexOfMonth: (token at: 2 ifAbsent: [ DateAndTime now monthName ]))
		day: (token at: 3 ifAbsent: [ Date current dayOfMonth ]) asInteger
		hour: (token at: 4 ifAbsent: [ Time now hour ]) asInteger
		minute: (token at: 5 ifAbsent: [ Time now minutes ]) asInteger
		second: (token at: 6 ifAbsent: [ Time now seconds ]) asInteger
		offset: (token at: 7 ifAbsent: [ 0 ]) asInteger hours asDuration! !

!PTStatus methodsFor: 'accessing' stamp: 'lr 6/11/2008 14:38'!
text
	^ text! !

!PTStatus methodsFor: 'accessing' stamp: 'lr 6/11/2008 14:37'!
text: aString
	text := aString! !

!PTStatus methodsFor: 'accessing' stamp: 'JohnMcKeon 2/27/2010 03:40'!
user
	^ user! !

!PTStatus methodsFor: 'accessing' stamp: 'JohnMcKeon 2/27/2010 03:40'!
user: anObject
	user := anObject! !

!PTTwitter class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:31'!
descriptionId
	^ MANumberDescription new
		xmlName: #id;
		accessor: #id;
		beRequired;
		yourself! !

!PTTwitter class methodsFor: 'initialzation' stamp: 'lr 10/28/2009 16:28'!
initialize
	StringConverter := TextConverter newForEncoding: #utf8! !

!PTTwitter class methodsFor: 'instance-creation' stamp: 'lr 6/11/2008 13:45'!
new
	^ self basicNew initialize! !

!PTTwitter class methodsFor: 'instance-creation' stamp: 'lr 6/11/2008 13:51'!
newFrom: anXml
	^ self new initializeFrom: anXml! !

!PTTwitter class methodsFor: 'accessing' stamp: 'lr 6/11/2008 14:25'!
xmlName
	^ nil! !

!PTTwitter methodsFor: 'comparing' stamp: 'lr 6/11/2008 13:41'!
= anObject
	^ self species = anObject species and: [ self id = anObject id ]! !

!PTTwitter methodsFor: 'actions' stamp: 'lr 7/9/2008 13:31'!
flush! !

!PTTwitter methodsFor: 'comparing' stamp: 'lr 7/9/2008 13:30'!
hash
	^ self id hash! !

!PTTwitter methodsFor: 'accessing' stamp: 'lr 6/11/2008 13:50'!
id
	^ id! !

!PTTwitter methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:31'!
id: anInteger
	id := anInteger! !

!PTTwitter methodsFor: 'initialize-release' stamp: 'lr 7/9/2008 13:41'!
initialize! !

!PTTwitter methodsFor: 'initialize-release' stamp: 'JohnMcKeon 2/27/2010 16:55'!
initializeFrom: anXml
	| value |
	self description do: [ :each |
		each xmlName notNil ifTrue: [
			value := anXml elementAt: each xmlName.
			value isNil
				ifTrue: [ each isRequired ifTrue: [ self error: each label , ' is a required field' ] ]
				ifFalse: [ each xmlName = #user
					ifTrue: [ self write: (PTUser newFrom: value) using: each]
					ifFalse: [
						self 
							write: (each fromString: (value contentString convertToWithConverter: StringConverter)) 
							using: each ] ] ] ]! !

PTTwitter subclass: #PTUser
	instanceVariableNames: 'username nickname summary location url image protected'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Twitter-Model'!

PTUser subclass: #PTAccount
	instanceVariableNames: 'email password followers friends timeline home useTree lastTweet prevLast'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Twitter-Model'!

!PTAccount methodsFor: 'private' stamp: 'JohnMcKeon 2/28/2010 11:58'!
call: anUrl class: aClass
	| response result |
	response := HTTPSocket
		httpGet: anUrl
		args: Dictionary new
		user: (email ifNil: [ String new ])
		passwd: (password ifNil: [ String new ]).
	response isString
		ifTrue: [ ^ nil ].
	result := OrderedCollection new.
	(XMLDOMParser parseDocumentFrom: response readStream)
		tagsNamed: aClass xmlName
		do: [ :each | result add: (aClass newFrom: each) ].
	^ result! !

!PTAccount methodsFor: 'accessing' stamp: 'lr 6/11/2008 13:38'!
email
	^ email! !

!PTAccount methodsFor: 'accessing' stamp: 'lr 6/11/2008 13:38'!
email: aString
	email := aString! !

!PTAccount methodsFor: 'actions' stamp: 'JohnMcKeon 2/27/2010 18:38'!
flush
	super flush.
	followers := friends := timeline := home := lastTweet := nil! !

!PTAccount methodsFor: 'querying' stamp: 'lr 7/9/2008 13:21'!
followers
	"Answer the authorized user's followers' current status."
	
	^ followers ifNil: [ followers := self call: 'http://twitter.com/statuses/followers.xml' class: PTUser ]! !

!PTAccount methodsFor: 'querying' stamp: 'lr 7/9/2008 13:21'!
friends
	"Answer the authorized user and their friends' current status."
	
	^ friends ifNil: [ friends := self call: 'http://twitter.com/statuses/friends.xml' class: PTUser ]! !

!PTAccount methodsFor: 'querying' stamp: 'JohnMcKeon 3/1/2010 16:35'!
home
	"This command returns the authorized user's home timeline."
	| twitter tweets |
	twitter := 'http://twitter.com/statuses/home_timeline.xml'.
	lastTweet isNil 
		ifFalse: [ twitter := twitter , '?since_id=' , lastTweet asString ].
	tweets := self call: twitter class: PTStatus.
	tweets ifNil: [ ^nil ].
	tweets isEmpty ifTrue: [^home].
	lastTweet isNil 
		ifFalse: [ tweets reversed do: [ :tweet | home addFirst: tweet ] ]
		ifTrue: [ home := tweets ].
	prevLast := lastTweet.
	lastTweet := tweets first id.
	^ home! !

!PTAccount methodsFor: 'accessing' stamp: 'JohnMcKeon 2/27/2010 18:30'!
lastTweet
	^lastTweet! !

!PTAccount methodsFor: 'accessing' stamp: 'lr 6/11/2008 13:38'!
password
	^ password! !

!PTAccount methodsFor: 'accessing' stamp: 'lr 6/11/2008 13:38'!
password: aString
	password := aString! !

!PTAccount methodsFor: 'querying' stamp: 'lr 7/9/2008 13:21'!
timeline
	"This command returns the authorized user's timeline."
	
	^ timeline ifNil: [ timeline := self call: 'http://twitter.com/statuses/user_timeline.xml' class: PTStatus ]! !

!PTAccount methodsFor: 'accessing' stamp: 'JohnMcKeon 2/27/2010 18:30'!
useTree
	^useTree! !

!PTUser class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:40'!
descriptionImage
	^ MAStringDescription new
		xmlName: #'profile_image_url';
		accessor: #image;
		yourself! !

!PTUser class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:38'!
descriptionLocation
	^ MAStringDescription new
		xmlName: #location;
		accessor: #location;
		yourself! !

!PTUser class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:41'!
descriptionNickname
	^ MAStringDescription new
		xmlName: #'screen_name';
		accessor: #nickname;
		yourself! !

!PTUser class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:42'!
descriptionProtected
	^ MABooleanDescription new
		xmlName: #protected;
		accessor: #protected;
		yourself! !

!PTUser class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:44'!
descriptionSummary
	^ MAMemoDescription new
		xmlName: #description;
		accessor: #summary;
		yourself! !

!PTUser class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:38'!
descriptionUrl
	^ MAStringDescription new
		xmlName: #url;
		accessor: #url;
		yourself! !

!PTUser class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:40'!
descriptionUsername
	^ MAStringDescription new
		xmlName: #name;
		accessor: #username;
		yourself! !

!PTUser class methodsFor: 'instance-creation' stamp: 'lr 7/9/2008 13:15'!
email: anEmailString password: aPasswordString
	^ PTAccount new
		email: anEmailString;
		password: aPasswordString;
		yourself! !

!PTUser class methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:29'!
xmlName
	^ #user! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
image
	^ image! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
image: anObject
	image := anObject! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
location
	^ location! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
location: anObject
	location := anObject! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
nickname
	^ nickname! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
nickname: anObject
	nickname := anObject! !

!PTUser methodsFor: 'printing' stamp: 'JohnMcKeon 2/27/2010 17:02'!
printOn: aStream
	nickname isNil 
		ifFalse: [ aStream nextPutAll: nickname ]
		ifTrue: [ username isNil 
			ifFalse: [ aStream nextPutAll: username ]
			ifTrue: [id isNil
				ifFalse: [ aStream nextPutAll: id ]
				ifTrue: [ super printOn: aStream ] ]]! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
protected
	^ protected! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
protected: anObject
	protected := anObject! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
summary
	^ summary! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
summary: anObject
	summary := anObject! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
url
	^ url! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
url: anObject
	url := anObject! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
username
	^ username! !

!PTUser methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:40'!
username: anObject
	username := anObject! !

!MADescription methodsFor: '*pier-twitter-model' stamp: 'lr 6/11/2008 13:49'!
xmlName
	^ self propertyAt: #xmlName ifAbsent: [ nil ]! !

!MADescription methodsFor: '*pier-twitter-model' stamp: 'lr 6/11/2008 13:49'!
xmlName: aString
	self propertyAt: #xmlName put: aString! !

PRWidget subclass: #PRTwitterClient
	instanceVariableNames: 'component'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Twitter-View'!

!PRTwitterClient class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:46'!
descriptionCount
	^ MASingleOptionDescription new
		label: 'Number of updates';
		options: (1 to: 20);
		accessor: #count;
		priority: 200;
		default: 3;
		yourself! !

!PRTwitterClient class methodsFor: 'accessing-descriptions' stamp: 'lr 7/23/2008 22:15'!
descriptionPassword
	^ MAPasswordDescription new
		accessor: #password;
		label: 'Password';
		priority: 110;
		default: '';
		yourself! !

!PRTwitterClient class methodsFor: 'accessing-descriptions' stamp: 'lr 10/21/2009 19:12'!
descriptionTimeout
	^ MADurationDescription new
		default: 10 minutes;
		accessor: #timeout;
		label: 'Timeout';
		priority: 120;
		beRequired;
		yourself! !

!PRTwitterClient class methodsFor: 'accessing-descriptions' stamp: 'lr 7/9/2008 13:47'!
descriptionUsername
	^ MAStringDescription new
		accessor: #username;
		label: 'Username';
		priority: 100;
		default: '';
		yourself! !

!PRTwitterClient class methodsFor: 'testing' stamp: 'lr 6/11/2008 08:28'!
isAbstract
	^ false! !

!PRTwitterClient class methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:47'!
label
	^ 'Twitter Client'! !

!PRTwitterClient methodsFor: 'accessing' stamp: 'lr 10/21/2009 19:42'!
account
	"If there is an account make sure it is still up-to-date, otherwise refresh it if necessary."
	
	self owner properties at: #'twitter.account' ifPresent: [ :account |
		self owner properties at: #'twitter.updated' ifPresent: [ :updated |
			(self username = account email
				and: [ self password = account password
				and: [ updated + self timeout > TimeStamp now ] ])
					ifFalse: [ self owner properties removeKey: #'twitter.account' ifAbsent: [ ] ] ] ].
	^ self owner properties at: #'twitter.account' ifAbsentPut: [
		self owner properties 
			at: #'twitter.updated'
			put: TimeStamp now.
		PTAccount
			email: self username
			password: self password ]! !

!PRTwitterClient methodsFor: 'private' stamp: 'lr 10/13/2009 16:26'!
agoString: aDateAndTime
	"Probably not that correct, maybe even entirely wrong. The goal is to be fuzzy, as much as possible. If you know better, then do better yourself and commit your changes."
	
	| duration minutes |
	aDateAndTime ifNil: [ ^ nil ].
	duration :=  DateAndTime now - aDateAndTime asLocal.
	minutes := duration asSeconds // 60.
	minutes <= 1
		ifTrue: [ ^ 'about one minute ago' ].
	minutes <= 50
		ifTrue: [ ^ minutes printString , ' minutes ago' ].
	minutes <= 120
		ifTrue: [ ^ 'about one hour ago' ].
	minutes <= 1080
		ifTrue: [ ^ (minutes // 60) printString , ' hours ago' ].
	minutes <= 2880
		ifTrue: [ ^ 'about one day ago' ].
	minutes <= 32000
		ifTrue: [ ^ (minutes // 1440) printString , ' days ago' ].
	minutes <= 86400
		ifTrue: [ ^ 'about one month ago' ].
	^ (minutes // 43200) printString , ' months ago'! !

!PRTwitterClient methodsFor: 'accessing' stamp: 'JohnMcKeon 2/28/2010 21:06'!
component
	^component ifNil: [ component := PTComponent new ]! !

!PRTwitterClient methodsFor: 'accessing' stamp: 'lr 6/11/2008 08:31'!
count
	^ self read: #descriptionCount! !

!PRTwitterClient methodsFor: 'accessing' stamp: 'lr 7/9/2008 13:54'!
password
	^ self read: #descriptionPassword! !

!PRTwitterClient methodsFor: 'rendering' stamp: 'JohnMcKeon 2/28/2010 21:29'!
renderContentOn: html
	| timeline |
	timeline := self account timeline 
		ifNil: [ ^ html heading level: 3; with: 'No results. Check username and password' ].
	self component tweets: (timeline first: (timeline size min: self count)).
	html render: component! !

!PRTwitterClient methodsFor: 'accessing' stamp: 'lr 10/21/2009 19:07'!
timeout
	^ self read: #descriptionTimeout! !

!PRTwitterClient methodsFor: 'accessing' stamp: 'lr 6/11/2008 09:35'!
username
	^ self read: #descriptionUsername! !

WAComponent subclass: #PTComponent
	instanceVariableNames: 'tweets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Twitter-Seaside'!

!PTComponent class methodsFor: 'instance creation' stamp: 'JohnMcKeon 2/28/2010 08:05'!
onTweets: aCollection
	^self new
		tweets: aCollection! !

!PTComponent methodsFor: 'initialize-release' stamp: 'JohnMcKeon 2/28/2010 08:04'!
initialize
	super initialize.
	tweets := OrderedCollection new! !

!PTComponent methodsFor: 'rendering' stamp: 'JohnMcKeon 2/28/2010 21:27'!
renderContentOn: html
	html unorderedList class: 'twitter'; with: [
		tweets do: [ :each | 
			html listItem: [
				self renderEntry: each on: html ] ] ].
! !

!PTComponent methodsFor: 'rendering' stamp: 'JohnMcKeon 2/28/2010 08:38'!
renderEntry: aStatus on: html
	html span class: 'status-body'; with: [
		html span
			class: 'entry-content';
			with: [ self renderEntryContent: aStatus on: html ].
		html span
			class: 'entry-meta';
			with: [ self renderEntryMeta: aStatus on: html ] ]! !

!PTComponent methodsFor: 'rendering' stamp: 'JohnMcKeon 2/28/2010 08:38'!
renderEntryContent: aStatus on: html
	| stream buffer char url |
	stream := aStatus text readStream.
	[ stream atEnd ] whileFalse: [
		(stream nextMatchAll: 'http://')
			ifTrue: [
				buffer := WriteStream on: String new.
				buffer nextPutAll: 'http://'.
				[ stream atEnd not and: [ stream peek isSafeForHTTP or: [ stream peek = $/ ] ] ]
					whileTrue: [ buffer nextPut: stream next ].
				html anchor url: buffer contents; with: buffer contents ]
			ifFalse: [
				char := stream next.
				(char = $@ or: [ char = $# ])
					ifTrue: [
						buffer := WriteStream on: String new.
						buffer nextPut: char.
						[ stream atEnd not and: [ (char := stream peek) isAlphaNumeric or: [ char = $/ or: [ char = $_ ] ] ] ]
							whileTrue: [ buffer nextPut: stream next ].
						url := WAUrl new hostname: 'twitter.com'.
						buffer contents first = $@
							ifTrue: [ url addToPath: buffer contents allButFirst ]
							ifFalse: [ url addToPath: 'search'; addParameter: 'q' value: buffer contents ].
						html anchor url: url; with: buffer contents ]
					ifFalse: [ html text: char ] ] ]! !

!PTComponent methodsFor: 'rendering' stamp: 'JohnMcKeon 3/1/2010 18:08'!
renderEntryMeta: aStatus on: html
	html anchor
		url: 'http://twitter.com/' , aStatus user username , '/statuses/' , aStatus id printString;
		with: aStatus agoString! !

!PTComponent methodsFor: 'rendering' stamp: 'JohnMcKeon 2/28/2010 08:37'!
renderTimelineOn: html
	html unorderedList class: 'twitter'; with: [
		tweets do: [ :each | 
			html listItem: [
				self renderEntry: each on: html ] ] ].
! !

!PTComponent methodsFor: 'accessing' stamp: 'JohnMcKeon 2/28/2010 08:06'!
tweets
	^ tweets! !

!PTComponent methodsFor: 'accessing' stamp: 'JohnMcKeon 2/28/2010 08:06'!
tweets: anObject
	tweets := anObject! !
PTTwitter initialize!
